// Copyright (c) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#ifndef ROCPRIM_DEVICE_DETAIL_CONFIG_DEVICE_SCAN_BY_KEY_HPP_
#define ROCPRIM_DEVICE_DETAIL_CONFIG_DEVICE_SCAN_BY_KEY_HPP_

#include "../../../type_traits.hpp"
#include "../device_config_helper.hpp"
#include <type_traits>

/* DO NOT EDIT THIS FILE
 * This file is automatically generated by `/scripts/autotune/create_optimization.py`.
 * so most likely you want to edit rocprim/device/device_(algo)_config.hpp
 */

/// \addtogroup primitivesmodule_deviceconfigs
/// @{

BEGIN_ROCPRIM_NAMESPACE

namespace detail
{

template<unsigned int arch, class key_type, class value_type, class enable = void>
struct default_scan_by_key_config : default_scan_by_key_config_base<key_type, value_type>
{};

// Based on key_type = double, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<128,
                         10,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = float, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         12,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = rocprim::half, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int64_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int64_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<128,
                         10,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         12,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = short, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = short, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int8_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx908),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         19,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = float, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         10,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         12,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = rocprim::half, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = rocprim::half, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         19,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int64_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int64_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         10,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         12,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = short, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         17,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int8_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx900),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         23,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = float, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         15,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         15,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = float, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = rocprim::half, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = rocprim::half, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = rocprim::half, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         23,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int64_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         15,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         15,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int8_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         13,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx906),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<128,
                         23,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         23,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         17,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = float, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         9,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = float, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         15,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = float, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         7,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = rocprim::half, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         23,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<128,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         13,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         19,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         9,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         7,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = short, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         9,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<128,
                         22,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<128,
                         17,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int8_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         12,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<128,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx1030),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<128,
                         10,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = float, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         12,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = rocprim::half, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int64_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int64_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<128,
                         10,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         12,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = short, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = short, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int8_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::unknown),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = double, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = double, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<128,
                         10,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = float, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         12,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = float, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = rocprim::half, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = rocprim::half, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int64_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int64_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<64,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int64_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 8)
                      && (sizeof(key_type) > 4) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<128,
                         10,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         12,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 4)
                      && (sizeof(key_type) > 2) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<128,
                         14,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 8)
                      && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = short, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 4)
                      && (sizeof(value_type) > 2))>>
    : scan_by_key_config<256,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = short, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 2)
                      && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = short, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 2)
                      && (sizeof(key_type) > 1) && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int64_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 8) && (sizeof(value_type) > 4))>>
    : scan_by_key_config<64,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = int
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 4) && (sizeof(value_type) > 2))>>
    : scan_by_key_config<128,
                         18,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::using_warp_scan>
{};

// Based on key_type = int8_t, value_type = short
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 2) && (sizeof(value_type) > 1))>>
    : scan_by_key_config<256,
                         20,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

// Based on key_type = int8_t, value_type = int8_t
template<class key_type, class value_type>
struct default_scan_by_key_config<
    static_cast<unsigned int>(target_arch::gfx90a),
    key_type,
    value_type,
    std::enable_if_t<(!bool(rocprim::is_floating_point<key_type>::value) && (sizeof(key_type) <= 1)
                      && (sizeof(value_type) <= 1))>>
    : scan_by_key_config<256,
                         24,
                         ::rocprim::block_load_method::block_load_transpose,
                         ::rocprim::block_store_method::block_store_transpose,
                         block_scan_algorithm::reduce_then_scan>
{};

} // end namespace detail

END_ROCPRIM_NAMESPACE

/// @}
// end of group primitivesmodule_deviceconfigs

#endif // ROCPRIM_DEVICE_DETAIL_CONFIG_DEVICE_SCAN_BY_KEY_HPP_
